package com.llisoft.pay.controller;

import java.io.IOException;
import java.nio.charset.StandardCharsets;

import javax.servlet.http.HttpServletRequest;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.util.StreamUtils;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;

import com.llisoft.pay.service.ApiService;
import com.wechat.pay.java.core.notification.RequestParam;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

@Api(tags="微信支付回调")
@Controller
@RequestMapping("/callback/wxpay")
public class CallbackWxPayController {
	
	private Logger logger = LoggerFactory.getLogger(CallbackWxPayController.class);

	@Autowired
	private ApiService apiService;
	
	
	@ApiOperation("异步通知 栋科")
	@PostMapping("/dongke/{pid}")
	public ResponseEntity<String> dongke(@PathVariable String pid, HttpServletRequest request) throws Exception {
		if(apiService.wxNotifyDongke(pid, this.notify(request))) {
			return ResponseEntity.ok().build();
		}
		return ResponseEntity.badRequest().build();
	}
	
	@ApiOperation("异步通知 青谷")
	@PostMapping("/qinggu/{pid}")
	public ResponseEntity<String> qinggu(@PathVariable String pid, HttpServletRequest request) throws Exception {
		if(apiService.wxNotifyQinggu(pid, this.notify(request))) {
			return ResponseEntity.ok().build();
		}
		return ResponseEntity.badRequest().build();
	}
	
	// https://github.com/wechatpay-apiv3/wechatpay-java/README.md
	private RequestParam notify(HttpServletRequest request) throws IOException {
		String signature = request.getHeader("wechatpay-signature"); // - HTTP 头 `Wechatpay-Signature`。应答的微信支付签名。
		String serial = request.getHeader("wechatpay-serial");		// - HTTP 头 `Wechatpay-Serial`。微信支付平台证书的序列号，验签必须使用序列号对应的微信支付平台证书。
		String nonce = request.getHeader("wechatpay-nonce");		// - HTTP 头 `Wechatpay-Nonce`。签名中的随机数。
		String timestamp = request.getHeader("wechatpay-timestamp"); // - HTTP 头 `Wechatpay-Timestamp`。签名中的时间戳。
//		String signatureType = headers.getFirst("wechatpay-signature-type"); // - HTTP 头 `Wechatpay-Signature-Type`。签名类型。
		// **切记使用原始报文**，不要用 JSON 对象序列化后的字符串，避免验签的 body 和原文不一致。
		String body = StreamUtils.copyToString(request.getInputStream(), StandardCharsets.UTF_8);
		logger.debug("收到微信支付异步通知: {}", body);
		return new RequestParam.Builder().serialNumber(serial)
			.nonce(nonce).signature(signature).timestamp(timestamp).body(body).build();
	}
	
}