package com.llisoft.pay.service;

import java.util.Map;
import java.util.Objects;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.llisoft.pay.common.Constant;
import com.llisoft.pay.config.MtaException;
import com.llisoft.pay.dao.PayDao;
import com.llisoft.pay.entity.Pay;
import com.llisoft.pay.util.CodeUtil;
import com.llisoft.pay.util.JsonUtil;
import com.llisoft.pay.vo.ApiInfoResponseVo;
import com.llisoft.pay.vo.ApiPayAliAppResponseVo;
import com.llisoft.pay.vo.ApiPayWxAppResponseVo;
import com.wechat.pay.java.core.notification.RequestParam;

@Service
public class ApiService {
	
	private Logger logger = LoggerFactory.getLogger(ApiService.class);
	
	
	@Autowired
	private PayDao payDao;
	@Autowired
	private PayWxV3Service payWxV3Service;
	@Autowired
	private PayAliNewService payAliNewService;

	
	/**
	 * 微信支付 APP 栋科
	 * @return
	 * @throws Exception 
	 */
	public ApiPayWxAppResponseVo wxAppDongke(String appid, int money, String title) throws Exception {
		Pay pay = this.add(money, title, Constant.TYPE_WX);
		logger.debug("添加支付记录：{}", JsonUtil.toJson(pay));
		return payWxV3Service.appDongke(appid, pay.getPid(), money, title);
	}
	
	/**
	 * 微信支付 APP 青谷
	 * @return
	 * @throws Exception 
	 */
	public ApiPayWxAppResponseVo wxAppQinggu(String appid, int money, String title) throws Exception {
		Pay pay = this.add(money, title, Constant.TYPE_WX);
		logger.debug("添加支付记录：{}", JsonUtil.toJson(pay));
		return payWxV3Service.appQinggu(appid, pay.getPid(), money, title);
	}
	
	
	/**
	 * 微信查询 栋科
	 * @return
	 * @throws Exception 
	 */
	public ApiInfoResponseVo wxInfoDongke(String pid) throws Exception {
		boolean payed = this.payed(pid);
		if(!payed) { // 待支付订单通过接口查询并更新支付状态
			if(payWxV3Service.queryDongke(pid)) { // 接口查询已支付
				payDao.update(pid, Constant.STATUS_PAYED, Constant.FLAG_QUERY);
				payed = true;
			}
		}
		return ApiInfoResponseVo.build(pid, payed);
	}
	
	/**
	 * 微信查询 青谷
	 * @return
	 * @throws Exception 
	 */
	public ApiInfoResponseVo wxInfoQinggu(String pid) throws Exception {
		boolean payed = this.payed(pid);
		if(!payed) { // 待支付订单通过接口查询并更新支付状态
			if(payWxV3Service.queryQinggu(pid)) { // 接口查询已支付
				payDao.update(pid, Constant.STATUS_PAYED, Constant.FLAG_QUERY);
				payed = true;
			}
		}
		return ApiInfoResponseVo.build(pid, payed);
	}
	
	/**
	 * 微信通知 栋科
	 * @param pid
	 * @param requestParam
	 * @return
	 * @throws Exception
	 */
	public boolean wxNotifyDongke(String pid, RequestParam requestParam) throws Exception {
		if(!this.payed(pid) && payWxV3Service.notifyDongke(requestParam)) {
			return payDao.update(pid, Constant.STATUS_PAYED, Constant.FLAG_NOTIFY);
		}
		return false;
	}
	
	/**
	 * 微信通知 青谷
	 * @param pid
	 * @param requestParam
	 * @return
	 * @throws Exception
	 */
	public boolean wxNotifyQinggu(String pid, RequestParam requestParam) throws Exception {
		if(!this.payed(pid) && payWxV3Service.notifyQinggu(requestParam)) {
			return payDao.update(pid, Constant.STATUS_PAYED, Constant.FLAG_NOTIFY);
		}
		return false;
	}

	
	/**
	 * 支付宝 APP 栋科
	 * @return
	 * @throws Exception 
	 */
	public ApiPayAliAppResponseVo aliAppDongke(int money, String title) throws Exception {
		Pay pay = this.add(money, title, Constant.TYPE_ALI);
		logger.debug("添加支付记录：{}", JsonUtil.toJson(pay));
		String text = payAliNewService.appDongke(pay.getPid(), money, title);
		ApiPayAliAppResponseVo responseVo = new ApiPayAliAppResponseVo();
		responseVo.setPid(pay.getPid());
		responseVo.setText(text);
		return responseVo;
	}
	
	/**
	 * 支付宝 APP 青谷
	 * @return
	 * @throws Exception 
	 */
	public ApiPayAliAppResponseVo aliAppQinggu(int money, String title) throws Exception {
		Pay pay = this.add(money, title, Constant.TYPE_ALI);
		logger.debug("添加支付记录：{}", JsonUtil.toJson(pay));
		String text = payAliNewService.appQinggu( pay.getPid(), money, title);
		ApiPayAliAppResponseVo responseVo = new ApiPayAliAppResponseVo();
		responseVo.setPid(pay.getPid());
		responseVo.setText(text);
		return responseVo;
	}
	
	
	/**
	 * 支付宝查询 栋科
	 * @return
	 * @throws Exception 
	 */
	public ApiInfoResponseVo aliInfoDongke(String pid) throws Exception {
		boolean payed = this.payed(pid);
		if(!payed) { // 待支付订单通过接口查询并更新支付状态
			if(payAliNewService.queryDongke(pid)) { // 接口查询已支付
				payDao.update(pid, Constant.STATUS_PAYED, Constant.FLAG_QUERY);
				payed = true;
			}
		}
		return ApiInfoResponseVo.build(pid, payed);
	}
	
	/**
	 * 支付宝查询 青谷
	 * @return
	 * @throws Exception 
	 */
	public ApiInfoResponseVo aliInfoQinggu(String pid) throws Exception {
		boolean payed = this.payed(pid);
		if(!payed) { // 待支付订单通过接口查询并更新支付状态
			if(payAliNewService.queryQinggu(pid)) { // 接口查询已支付
				payDao.update(pid, Constant.STATUS_PAYED, Constant.FLAG_QUERY);
				payed = true;
			}
		}
		return ApiInfoResponseVo.build(pid, payed);
	}
	
	/**
	 * 支付宝通知
	 * @param pid
	 * @param requestParam
	 * @return
	 * @throws Exception
	 */
	public boolean aliNotify(String pid, Map<String, String> paramMap) throws Exception {
		if(!this.payed(pid) && payAliNewService.notify(paramMap)) {
			return payDao.update(pid, Constant.STATUS_PAYED, Constant.FLAG_NOTIFY);
		}
		return false;
	}
	

	/**
	 * 添加支付记录
	 * @return
	 * @throws Exception 
	 */
	private Pay add(int money, String title, int type) throws Exception {
		Pay pay = new Pay();
		pay.setMoney(money);
		pay.setTitle(title);
		pay.setType(type);
		pay.setPid(CodeUtil.time());
		pay.setStatus(Constant.STATUS_WAIT);
		payDao.insert(pay);
		logger.debug("添加支付记录：{}", JsonUtil.toJson(pay));
		return pay;
	}
	
	/**
	 * 查询支付记录
	 * @return
	 * @throws Exception 
	 */
	private boolean payed(String pid) throws Exception {
		Pay pay = payDao.selectByPid(pid);
		if(Objects.isNull(pay)) {
			throw MtaException.build("支付记录不存在：" + pid);
		}
		return pay.getStatus() == Constant.STATUS_PAYED;
	}
	
}